@@ -23,7 +23,6 @@ import math
 import os
 import pickle
 import shutil
-import struct
 import textwrap
 import threading
 import time
@@ -35,8 +34,8 @@ from cachetools import TTLCache
 from streamlit import config
 from streamlit import file_util
 from streamlit import util
+from streamlit.error_util import handle_uncaught_app_exception
 from streamlit.errors import StreamlitAPIWarning
-from streamlit.errors import StreamlitDeprecationWarning
 from streamlit.hashing import Context
 from streamlit.hashing import update_hash
 from streamlit.hashing import HashReason
@@ -55,7 +54,7 @@ _CacheEntry = namedtuple("_CacheEntry", ["value", "hash"])
 _DiskCacheEntry = namedtuple("_DiskCacheEntry", ["value"])
 
 
-class _MemCaches(object):
+class _MemCaches:
     """Manages all in-memory st.cache caches"""
 
     def __init__(self):
@@ -63,6 +62,9 @@ class _MemCaches(object):
         self._lock = threading.RLock()
         self._function_caches = {}  # type: Dict[str, TTLCache]
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def get_cache(
         self, key: str, max_entries: Optional[float], ttl: Optional[float]
     ) -> TTLCache:
@@ -120,6 +122,9 @@ class ThreadLocalCacheInfo(threading.local):
         self.cached_func_stack = []
         self.suppress_st_function_warning = 0
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
 
 _cache_info = ThreadLocalCacheInfo()
 
@@ -185,6 +190,9 @@ class _AddCopy(ast.NodeTransformer):
     def __init__(self, func_name):
         self.func_name = func_name
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def visit_Call(self, node):
         if (
             hasattr(node.func, "func")
@@ -323,7 +331,7 @@ def _read_from_cache(
         )
 
     except CachedObjectMutationError as e:
-        st.exception(CachedObjectMutationWarning(e))
+        handle_uncaught_app_exception(CachedObjectMutationWarning(e))
         return e.cached_value
 
     except CacheKeyNotFoundError as e:
@@ -460,55 +468,7 @@ def cache(
             ttl=ttl,
         )
 
-    # Create the unique key for this function's cache. The cache will be
-    # retrieved from inside the wrapped function.
-    #
-    # A naive implementation would involve simply creating the cache object
-    # right here in the wrapper, which in a normal Python script would be
-    # executed only once. But in Streamlit, we reload all modules related to a
-    # user's app when the app is re-run, which means that - among other
-    # things - all function decorators in the app will be re-run, and so any
-    # decorator-local objects will be recreated.
-    #
-    # Furthermore, our caches can be destroyed and recreated (in response
-    # to cache clearing, for example), which means that retrieving the
-    # function's cache here (so that the wrapped function can save a lookup)
-    # is incorrect: the cache itself may be recreated between
-    # decorator-evaluation time and decorated-function-execution time. So
-    # we must retrieve the cache object *and* perform the cached-value lookup
-    # inside the decorated function.
-
-    func_hasher = hashlib.new("md5")
-
-    # Include the function's __module__ and __qualname__ strings in the hash.
-    # This means that two identical functions in different modules
-    # will not share a hash; it also means that two identical *nested*
-    # functions in the same module will not share a hash.
-    # We do not pass `hash_funcs` here, because we don't want our function's
-    # name to get an unexpected hash.
-    update_hash(
-        (func.__module__, func.__qualname__),
-        hasher=func_hasher,
-        hash_funcs=None,
-        hash_reason=HashReason.CACHING_FUNC_BODY,
-        hash_source=func,
-    )
-
-    # Include the function's body in the hash. We *do* pass hash_funcs here,
-    # because this step will be hashing any objects referenced in the function
-    # body.
-    update_hash(
-        func,
-        hasher=func_hasher,
-        hash_funcs=hash_funcs,
-        hash_reason=HashReason.CACHING_FUNC_BODY,
-        hash_source=func,
-    )
-
-    cache_key = func_hasher.hexdigest()
-    _LOGGER.debug(
-        "mem_cache key for %s.%s: %s", func.__module__, func.__qualname__, cache_key
-    )
+    cache_key = None
 
     @functools.wraps(func)
     def wrapped_func(*args, **kwargs):
@@ -528,6 +488,15 @@ def cache(
             message = "Running `%s(...)`." % name
 
         def get_or_create_cached_value():
+            nonlocal cache_key
+            if cache_key is None:
+                # Delay generating the cache key until the first call.
+                # This way we can see values of globals, including functions
+                # defined after this one.
+                # If we generated the key earlier we would only hash those
+                # globals by name, and miss changes in their code or value.
+                cache_key = _hash_func(func, hash_funcs)
+
             # First, get the cache that's attached to this function.
             # This cache's key is generated (above) from the function's code.
             mem_cache = _mem_caches.get_cache(cache_key, max_entries, ttl)
@@ -615,6 +584,57 @@ def cache(
     return wrapped_func
 
 
+def _hash_func(func, hash_funcs) -> str:
+    # Create the unique key for a function's cache. The cache will be retrieved
+    # from inside the wrapped function.
+    #
+    # A naive implementation would involve simply creating the cache object
+    # right in the wrapper, which in a normal Python script would be executed
+    # only once. But in Streamlit, we reload all modules related to a user's
+    # app when the app is re-run, which means that - among other things - all
+    # function decorators in the app will be re-run, and so any decorator-local
+    # objects will be recreated.
+    #
+    # Furthermore, our caches can be destroyed and recreated (in response to
+    # cache clearing, for example), which means that retrieving the function's
+    # cache in the decorator (so that the wrapped function can save a lookup)
+    # is incorrect: the cache itself may be recreated between
+    # decorator-evaluation time and decorated-function-execution time. So we
+    # must retrieve the cache object *and* perform the cached-value lookup
+    # inside the decorated function.
+    func_hasher = hashlib.new("md5")
+
+    # Include the function's __module__ and __qualname__ strings in the hash.
+    # This means that two identical functions in different modules
+    # will not share a hash; it also means that two identical *nested*
+    # functions in the same module will not share a hash.
+    # We do not pass `hash_funcs` here, because we don't want our function's
+    # name to get an unexpected hash.
+    update_hash(
+        (func.__module__, func.__qualname__),
+        hasher=func_hasher,
+        hash_funcs=None,
+        hash_reason=HashReason.CACHING_FUNC_BODY,
+        hash_source=func,
+    )
+
+    # Include the function's body in the hash. We *do* pass hash_funcs here,
+    # because this step will be hashing any objects referenced in the function
+    # body.
+    update_hash(
+        func,
+        hasher=func_hasher,
+        hash_funcs=hash_funcs,
+        hash_reason=HashReason.CACHING_FUNC_BODY,
+        hash_source=func,
+    )
+    cache_key = func_hasher.hexdigest()
+    _LOGGER.debug(
+        "mem_cache key for %s.%s: %s", func.__module__, func.__qualname__, cache_key
+    )
+    return cache_key
+
+
 class Cache(Dict[Any, Any]):
     """Cache object to persist data across reruns.
 
@@ -650,6 +670,9 @@ class Cache(Dict[Any, Any]):
 
         dict.__init__(self)
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def has_changes(self) -> bool:
         current_frame = inspect.currentframe()
 
@@ -811,6 +834,9 @@ class CachedObjectMutationError(ValueError):
         else:
             self.cached_func_name = _get_cached_func_name_md(func_or_code)
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
 
 class CachedStFunctionWarning(StreamlitAPIWarning):
     def __init__(self, st_func_name, cached_func):
