@@ -23,6 +23,7 @@ import tornado.web
 
 import streamlit.server.routes
 from streamlit import type_util
+from streamlit import util
 from streamlit.elements.utils import register_widget, NoValue
 from streamlit.errors import StreamlitAPIException
 from streamlit.logger import get_logger
@@ -58,6 +59,9 @@ class CustomComponent:
         self.path = path
         self.url = url
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     @property
     def abspath(self) -> Optional[str]:
         """The absolute path that the component is served from."""
@@ -320,9 +324,9 @@ class ComponentRequestHandler(tornado.web.RequestHandler):
         LOGGER.debug("ComponentRequestHandler: GET: %s -> %s", path, abspath)
 
         try:
-            with open(abspath, "r") as file:
+            with open(abspath, "r", encoding="utf-8") as file:
                 contents = file.read()
-        except OSError as e:
+        except (OSError, UnicodeDecodeError) as e:
             self.write(f"{path} read error: {e}")
             self.set_status(404)
             return
@@ -400,6 +404,9 @@ class ComponentRegistry:
         self._components = {}  # type: Dict[str, CustomComponent]
         self._lock = threading.Lock()
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def register_component(self, component: CustomComponent) -> None:
         """Register a CustomComponent.
 
