@@ -22,7 +22,7 @@ from .utils import register_widget
 
 
 class ColorPickerMixin:
-    def color_picker(self, label, value=None, key=None):
+    def color_picker(self, label, value=None, key=None, help=None):
         """Display a color picker widget.
 
         Parameters
@@ -37,6 +37,8 @@ class ColorPickerMixin:
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
+        help : str
+            A tooltip that gets displayed next to the color picker.
 
         Returns
         -------
@@ -78,6 +80,8 @@ class ColorPickerMixin:
         color_picker_proto = ColorPickerProto()
         color_picker_proto.label = label
         color_picker_proto.default = str(value)
+        if help is not None:
+            color_picker_proto.help = help
 
         ui_value = register_widget("color_picker", color_picker_proto, user_key=key)
         current_value = ui_value if ui_value is not None else value
