@@ -78,7 +78,7 @@ class MarkdownMixin:
 
         return self.dg._enqueue("markdown", markdown_proto)
 
-    def header(self, body):
+    def header(self, body, anchor=None):
         """Display text in header formatting.
 
         Parameters
@@ -86,6 +86,10 @@ class MarkdownMixin:
         body : str
             The text to display.
 
+        anchor : str
+            The anchor name of the header that can be accessed with #anchor
+            in the URL. If omitted, it generates an anchor using the body.
+
         Example
         -------
         >>> st.header('This is a header')
@@ -96,10 +100,14 @@ class MarkdownMixin:
 
         """
         header_proto = MarkdownProto()
-        header_proto.body = "## %s" % clean_text(body)
+        if anchor is None:
+            header_proto.body = f"## {clean_text(body)}"
+        else:
+            header_proto.body = f'<h2 data-anchor="{anchor}">{clean_text(body)}</h2>'
+            header_proto.allow_html = True
         return self.dg._enqueue("markdown", header_proto)
 
-    def subheader(self, body):
+    def subheader(self, body, anchor=None):
         """Display text in subheader formatting.
 
         Parameters
@@ -107,6 +115,10 @@ class MarkdownMixin:
         body : str
             The text to display.
 
+        anchor : str
+            The anchor name of the header that can be accessed with #anchor
+            in the URL. If omitted, it generates an anchor using the body.
+
         Example
         -------
         >>> st.subheader('This is a subheader')
@@ -117,7 +129,12 @@ class MarkdownMixin:
 
         """
         subheader_proto = MarkdownProto()
-        subheader_proto.body = "### %s" % clean_text(body)
+        if anchor is None:
+            subheader_proto.body = f"### {clean_text(body)}"
+        else:
+            subheader_proto.body = f'<h3 data-anchor="{anchor}">{clean_text(body)}</h3>'
+            subheader_proto.allow_html = True
+
         return self.dg._enqueue("markdown", subheader_proto)
 
     def code(self, body, language="python"):
@@ -153,7 +170,7 @@ class MarkdownMixin:
         code_proto.body = clean_text(markdown)
         return self.dg._enqueue("markdown", code_proto)
 
-    def title(self, body):
+    def title(self, body, anchor=None):
         """Display text in title formatting.
 
         Each document should have a single `st.title()`, although this is not
@@ -164,6 +181,10 @@ class MarkdownMixin:
         body : str
             The text to display.
 
+        anchor : str
+            The anchor name of the header that can be accessed with #anchor
+            in the URL. If omitted, it generates an anchor using the body.
+
         Example
         -------
         >>> st.title('This is a title')
@@ -174,7 +195,11 @@ class MarkdownMixin:
 
         """
         title_proto = MarkdownProto()
-        title_proto.body = "# %s" % clean_text(body)
+        if anchor is None:
+            title_proto.body = f"# {clean_text(body)}"
+        else:
+            title_proto.body = f'<h1 data-anchor="{anchor}">{clean_text(body)}</h1>'
+            title_proto.allow_html = True
         return self.dg._enqueue("markdown", title_proto)
 
     def latex(self, body):
