@@ -32,12 +32,13 @@ class NumberInputMixin:
         step=None,
         format=None,
         key=None,
+        help=None,
     ):
         """Display a numeric input widget.
 
         Parameters
         ----------
-        label : str or None
+        label : str
             A short label explaining to the user what this input is for.
         min_value : int or float or None
             The minimum permitted value.
@@ -61,6 +62,8 @@ class NumberInputMixin:
             If this is omitted, a key will be generated for the widget
             based on its content. Multiple widgets of the same type may
             not share the same key.
+        help : str
+            A tooltip that gets displayed next to the input.
 
         Returns
         -------
@@ -201,6 +204,8 @@ class NumberInputMixin:
         )
         number_input_proto.label = label
         number_input_proto.default = value
+        if help is not None:
+            number_input_proto.help = help
 
         if min_value is not None:
             number_input_proto.min = min_value
