@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from streamlit import util
 from streamlit import file_util
 from streamlit import config
 
@@ -63,6 +64,9 @@ class FolderBlackList(object):
         self._folder_blacklist = list(folder_blacklist)
         self._folder_blacklist.extend(DEFAULT_FOLDER_BLACKLIST)
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def is_blacklisted(self, filepath):
         """Test if filepath is in the blacklist.
 
