@@ -20,6 +20,7 @@ import hashlib
 
 from streamlit.report_thread import get_report_ctx
 from streamlit.logger import get_logger
+from streamlit import util
 
 LOGGER = get_logger(__name__)
 
@@ -65,6 +66,9 @@ class MediaFile(object):
         self._content = content
         self._mimetype = mimetype
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     @property
     def url(self):
         return "{}/{}.{}".format(
@@ -116,6 +120,9 @@ class MediaFileManager(object):
             dict
         )  # type: DefaultDict[str, Dict[str, MediaFile]]
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def del_expired_files(self):
         LOGGER.debug("Deleting expired files...")
 
