@@ -22,6 +22,8 @@ from blinker import Signal
 from streamlit import config
 from streamlit import magic
 from streamlit import source_util
+from streamlit import util
+from streamlit.error_util import handle_uncaught_app_exception
 from streamlit.media_file_manager import media_file_manager
 from streamlit.report_thread import ReportThread
 from streamlit.report_thread import get_report_ctx
@@ -123,6 +125,9 @@ class ScriptRunner(object):
         # This is initialized in start()
         self._script_thread = None
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def start(self):
         """Start a new thread to process the ScriptEventQueue.
 
@@ -338,16 +343,11 @@ class ScriptRunner(object):
             pass
 
         except BaseException as e:
-            # Show exceptions in the Streamlit report.
-            LOGGER.debug(e)
-            import streamlit as st
-
-            st.exception(e)  # This is OK because we're in the script thread.
-            # TODO: Clean up the stack trace, so it doesn't include
-            # ScriptRunner.
+            handle_uncaught_app_exception(e)
 
         finally:
             self._widgets.reset_triggers()
+            self._widgets.cull_nonexistent(ctx.widget_ids_this_run.items())
             self.on_event.send(ScriptRunnerEvent.SCRIPT_STOPPED_WITH_SUCCESS)
             # delete expired files now that the script has run and files in use
             # are marked as active
@@ -386,6 +386,9 @@ class RerunException(ScriptControlException):
         """
         self.rerun_data = rerun_data
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
 
 def _clean_problem_modules():
     """Some modules are stateful, so we have to clear their state."""
@@ -422,6 +425,9 @@ class modified_sys_path(object):
         self._report = report
         self._added_path = False
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     def __enter__(self):
         if self._report.script_path not in sys.path:
             sys.path.insert(0, self._report.script_path)
