@@ -22,6 +22,7 @@ from tornado import gen
 from tornado import locks
 
 import streamlit
+from streamlit import util
 from streamlit import errors
 
 from streamlit.logger import get_logger
@@ -49,6 +50,9 @@ class AbstractStorage(object):
         )
         self._write_lock = locks.Lock()
 
+    def __repr__(self) -> str:
+        return util.repr_(self)
+
     @gen.coroutine
     def save_report_files(
         self, report_id, files, progress_coroutine=None, manifest_save_order=None
